/*---------------------------------------------------------------------------*
  File:     prof_api.h

  Copyright (C) Nintendo.  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo of America Inc. and/or Nintendo
  Company Ltd., and are protected by Federal copyright law.  They may
  not be disclosed to third parties or copied or duplicated in any form,
  in whole or in part, without the prior written consent of Nintendo.

 *---------------------------------------------------------------------------*/

#ifndef NN_PROF_PROF_API_H_
#define NN_PROF_PROF_API_H_


#include <nn/config.h>	/* For NN_IS_UNUSED_VAR */

/* \defgroup profilerapi API */

/*! \addtogroup status */
/*! \{ */
/*! \defgroup statuses Status Values

    These definitions are used to denote the status of the profiler.

    \sa nnprofGetProfilerStatus

    \ingroup status
*/
/*! \{ */
#define NN_PROFILER_STATUS_OFFLINE          0x0 /*!< \brief The profiler is not running.*/
#define NN_PROFILER_STATUS_NOT_PROFILING    0x1 /*!< \brief The profiler is running, but not taking a profile.*/
#define NN_PROFILER_STATUS_IS_PROFILING     0x2 /*!< \brief The profiler is running and taking a profile.*/
#define NN_PROFILER_STATUS_TRANSFERRING     0x3 /*!< \brief The profiler is currently transferring profile data to the PC.*/
/*! \} */
/*! \} */


#ifdef __cplusplus
extern "C" {
#endif

/*! \defgroup codeblocks Code Blocks
    \brief Code blocks allow for the manual instrumentation of blocks of code.
*/
/*! \brief Creates a code block enumeration which the profiler GUI can find in the DWARF data.

    This function registers the names Code Blocks (the names that will appear on the _Instrumented_ tab) in your ELF.
    The names are then pulled out of the ELF DWARF data by the PC to show names of code blocks.
    This is to conserve space in the recorded data when a code block is hit.
    If this API is not used, Code Blocks can still be used but rather than a name, they will only be displayed by a number on the PC.

    All code block names should be entered in a list, as if this were an enum declaration.
    For example:
    \code
    NN_PROF_DEFINE_CODEBLOCKNAMES_ENUM(
        CodeBlock_Name1,
        CodeBlock_Name2,
        CodeBlock_Name3);
    \endcode

    \hideinitializer

    \ingroup codeblocks
*/
#define NN_PROF_DEFINE_CODEBLOCKNAMES_ENUM(...) \
    enum NNPROF_CODEBLOCKNAMES { __VA_ARGS__ }; \
    void nnprofCodeBlockEnter(NNPROF_CODEBLOCKNAMES name);\
    inline void nnprofCodeBlockEnter(NNPROF_CODEBLOCKNAMES name NN_IS_UNUSED_VAR){nnprofdetailCodeBlockEnter(name);}\
    void nnprofCodeBlockExit(NNPROF_CODEBLOCKNAMES name);\
    inline void nnprofCodeBlockExit(NNPROF_CODEBLOCKNAMES name NN_IS_UNUSED_VAR){nnprofdetailCodeBlockExit(name);}\
    void nnprofCodeBlockExit(NNPROF_CODEBLOCKNAMES name)

/*! \enum NNPROF_CODEBLOCKNAMES
    \brief An enumeration created by the \ref NN_PROF_DEFINE_CODEBLOCKNAMES_ENUM macro.

    This enumeration contains all of the code block names that will be imported by the PC.

    \ingroup codeblocks
*/

/*! \fn void nnprofCodeBlockEnter(NNPROF_CODEBLOCKNAMES name)
    \brief This function marks the beginning of a code block.

    \param name One of the enumerator definitions defined via the \ref NN_PROF_DEFINE_CODEBLOCKNAMES_ENUM macro.

    \ingroup codeblocks
*/

/*! \fn void nnprofCodeBlockExit(NNPROF_CODEBLOCKNAMES name)
    \brief This function marks the end of a code block.

    \param name One of the enumerator definitions defined via the \ref NN_PROF_DEFINE_CODEBLOCKNAMES_ENUM macro.

    \ingroup codeblocks
*/

#if !defined(NN_PROF_DO_NOT_USE)

/* ------------------------------------------------------------------------- */
/*! \defgroup marking Frame Marking

    \brief The frame marking functionality allows for the placing of frame data when looking at the profile data in the GUI.
*/
/*! \brief This function will place a marker in the profile buffer indicating the top of the game main loop allowing the profiler to detect frames.

    This function is intended to denote frames.

    \ingroup marking
*/
extern void nnprofRecordTopMainLoop(void);

/*! \brief This function will place a marker in the profile buffer indicating that the application is waiting for VBlank to occur.

    \ingroup marking
*/
extern void nnprofRecordWaitForVBlank(void);
/* ------------------------------------------------------------------------- */

/* ------------------------------------------------------------------------- */
/*! \defgroup status Profiler Status 

    \brief Functions and values that are used to determine the profiler status.
*/
/*! \brief This function is used to determine the current profiler status.

    Possible values that are returned from this function are described in \ref statuses.

    \ingroup status
*/
extern unsigned int nnprofGetProfilerStatus(void);
/* ------------------------------------------------------------------------- */

/*--
Internal functions : Recommended not to be used directly */
extern void nnprofdetailCodeBlockEnter(unsigned int);
extern void nnprofdetailCodeBlockExit(unsigned int);
/*--*/

#else /* if !defined(NN_PROF_DO_NOT_USE) */

#define nnprofRecordTopMainLoop()           ((void)0)
#define nnprofRecordWaitForVBlank()         ((void)0)
#define nnprofGetProfilerStatus()           ((unsigned int)0)

#define nnprofdetailCodeBlockEnter(...)     ((void)0)
#define nnprofdetailCodeBlockExit(...)      ((void)0)
#endif

#ifdef __cplusplus
}
#endif


#endif /* NN_PROF_PROF_API_H_ */
